import React, { useEffect, useState } from 'react';
import {
  useAppDispatch,
  useAuthorizationCache,
  useML,
  useQuerybuilderGraph,
  useQuerybuilderSettings,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { resetGraphQueryResults, queryingBackend } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Navbar } from '../components/navbar/navbar';
import { Resizable } from '@graphpolaris/shared/lib/components/Resizable';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { EventBus } from '@graphpolaris/shared/lib/data-access/api/eventBus';
import { Onboarding } from '../components/onboarding/onboarding';
import { wsQueryRequest } from '@graphpolaris/shared/lib/data-access/broker';
import { URLParams, setParam } from '@graphpolaris/shared/lib/data-access/api/url';
import { VisualizationPanel } from '@graphpolaris/shared/lib/vis';
import { QueryBuilder } from '@graphpolaris/shared/lib/querybuilder';
import { SideNavTab, Sidebar } from '@graphpolaris/shared/lib/sidebar';
import { VisualizationManager } from '@graphpolaris/shared/lib/vis/manager';
import { ConfigPanel } from '@graphpolaris/shared/lib/vis/components/config';
import { Tooltip, TooltipTrigger, Button, TooltipContent, TooltipProvider } from '@graphpolaris/shared';
import { ControlContainer } from '@graphpolaris/shared/lib/components/controls';
import { Searchbar } from '@graphpolaris/shared/lib/sidebar/search/searchbar';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { Fullscreen, FitScreen, Remove, Schema as SchemaIcon, Search as SearchIcon } from '@mui/icons-material';

export type App = {
  load?: string;
};

export function App(props: App) {
  const auth = useAuthorizationCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const ml = useML();
  const session = useSessionCache();
  const dispatch = useAppDispatch();
  const queryBuilderSettings = useQuerybuilderSettings();
  const manager = VisualizationManager();

  const runQuery = () => {
    if (session?.currentSaveState && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        dispatch(queryingBackend());
        wsQueryRequest(Query2BackendQuery(session.currentSaveState, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    if (props.load) {
      setParam(URLParams.saveState, props.load);
    }
  }, [props]);

  const [authCheck, setAuthCheck] = useState(false);
  const [tab, setTab] = useState<SideNavTab>('Schema');
  const [visFullSize, setVisFullSize] = useState<boolean>(false);

  return (
    <div className="h-screen w-screen overflow-clip">
      <EventBus
        onRunQuery={runQuery}
        onAuthorized={() => {
          setAuthCheck(true);
        }}
      />
      {authCheck && (
        <>
          <Onboarding />
          <DashboardAlerts />
          <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
            <div className="flex flex-col h-screen max-h-screen relative">
              <aside className="absolute w-full h-12">
                <Navbar />
              </aside>
              <main className="grow flex flex-row h-screen pt-12">
                <Sidebar onTab={(tab) => setTab(tab)} />
                <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
                  {tab !== undefined ? (
                    <div className="flex flex-col border w-full h-full bg-light">
                      <div className="sticky shrink-0 top-0 flex items-stretch justify-between h-7 bg-secondary-100 border-b border-secondary-200 max-w-full">
                        <div className="flex items-center">
                          <h1 className="text-xs font-semibold text-secondary-600 px-2 truncate">{tab}</h1>
                        </div>
                        <div className="shrink-0 sticky right-0 px-0.5 ml-auto items-center flex">
                          <ControlContainer>
                            <TooltipProvider delayDuration={100}>
                              <Tooltip>
                                <TooltipTrigger asChild>
                                  <Button type="secondary" variant="ghost" size="xs" iconComponent={<Remove />} onClick={() => {}} />
                                </TooltipTrigger>
                                <TooltipContent side={'top'}>
                                  <p>Hide</p>
                                </TooltipContent>
                              </Tooltip>
                              {tab === 'Schema' && (
                                <Tooltip>
                                  <TooltipTrigger asChild>
                                    <Button type="secondary" variant="ghost" size="xs" iconComponent={<FitScreen />} onClick={() => {}} />
                                  </TooltipTrigger>
                                  <TooltipContent side={'top'}>
                                    <p>Fit to screen</p>
                                  </TooltipContent>
                                </Tooltip>
                              )}
                              {tab === 'Search' && (
                                <Tooltip>
                                  <TooltipTrigger asChild>
                                    <Button type="secondary" variant="ghost" size="xs" iconComponent={<Fullscreen />} onClick={() => {}} />
                                  </TooltipTrigger>
                                  <TooltipContent side={'top'}>
                                    <p>Mock icon</p>
                                  </TooltipContent>
                                </Tooltip>
                              )}
                            </TooltipProvider>
                          </ControlContainer>
                        </div>
                      </div>
                      {tab === 'Search' && <Searchbar />}
                      {tab === 'Schema' && <Schema auth={authCheck} />}
                    </div>
                  ) : null}

                  <Resizable divisorSize={3} horizontal={false}>
                    <VisualizationPanel
                      manager={manager}
                      fullSize={() => {
                        setVisFullSize(!visFullSize);
                        tab === undefined && setTab('Schema');
                        tab !== undefined && setTab(undefined);
                      }}
                    />

                    <QueryBuilder onRunQuery={runQuery} />
                  </Resizable>
                </Resizable>
                <div className="info-panel flex h-full w-60 ml-[3px] shrink-0 overflow-auto bg-light border">
                  <ConfigPanel manager={manager} />
                </div>
              </main>
            </div>
          </div>
        </>
      )}
    </div>
  );
}

export default App;
